import json
from pathlib import Path
import epdx
import matplotlib.pyplot as plt
import numpy as np
from epdx import ImpactCategory
from shared import download_to_epdx, get_folder


def read_epdx_files(download_folder: Path) -> list[epdx.EPD]:
    """Reads a folder of EPDx formatted json files in to EPDx Pydantic classes"""

    epdx_data = []
    for file in download_folder.iterdir():
        if str(file).endswith('.epdx.json'):
            epdx_data.append(epdx.EPD(**json.loads(file.read_text())))

    return epdx_data


def visualize_histogram(impact_category_values: dict[str, list]):
    """Histogram of GWP A1-A3 Values"""

    fig, ax = plt.subplots()
    gwp_a1a3 = [value for value in impact_category_values.get("a1a3") if value]
    ax.hist(gwp_a1a3)

    ax.set_title("GWP values for A1-A3 per declared unit")
    ax.set_xlabel("kg-CO2-Eq. per declared unit")

    plt.show()


def organize_data(epdx_data: list[epdx.EPD]) -> dict[str, list]:
    """Organized data for charts"""

    impact_category_values = {key: [] for key in ImpactCategory().__fields__.keys()}

    for epd in epdx_data:
        for phase in impact_category_values.keys():
            try:
                gwp_values = getattr(epd.gwp, phase) or 0.0
            except AttributeError:
                gwp_values = 0.0

            impact_category_values[phase].append(gwp_values)

    return impact_category_values


def visualize_epds(download_folder: Path, search_key: str):
    """Get EPDs and Visualize them as a stacked bar chart and a histogram"""

    download_to_epdx(download_folder, search_key)
    epdx_data = read_epdx_files(download_folder)

    impact_category_values = organize_data(epdx_data)
    visualize_histogram(impact_category_values)


if __name__ == "__main__":
    _folder = get_folder(__file__, "data")
    visualize_epds(_folder, "beton")
